DebugLogBackend = LibStub('AceAddon-3.0'):NewAddon("DebugLogBackend", "AceConsole-3.0")
local DebugLogBackend = DebugLogBackend

local tostring, date, tinsert, tconcat = tostring, date, table.insert, table.concat

local LibDebugLog = LibStub('LibDebugLog-1.0')

local db
local frame

do
	local prefix = {}
	function DebugLogBackend:MessageLogged(event, source, timestamp, message)
		if not frame then return end
		for i = 1,#prefix do prefix[i] = nil end
		if db.showPrefix then
			tinsert(prefix, "(DEBUG)")
		end
		if db.showSource then
			tinsert(prefix, tostring(source))
		end
		if db.showTimestamp then
			tinsert(prefix, ("[%s.%3d]"):format(date("%H:%M:%S", timestamp), (timestamp % 1) * 1000))
		end
		if #prefix > 0 then
			message = ("|cff7fff7f%s|r: %s"):format(tconcat(prefix, " "), message)
		end
		frame:AddMessage(message)
	end
end

function DebugLogBackend:UpdateFrame()
	frame = _G[db.chatFrame == 0 and "DEFAULT_CHAT_FRAME" or "ChatFrame"..db.chatFrame]
end

function DebugLogBackend:OnInitialize()
	self.db = LibStub('AceDB-3.0'):New("DebugLogBackendDB", {
		global = {
			enabled = true,
			showPrefix = true,
			showSource = true,
			showTimestamp = true,
			chatFrame = 0,
		}
	})
	db = self.db.global
	self:SetEnabledState(db.enabled)
	
	local options = self:BuildOptions(db)
	LibStub("AceConfig-3.0"):RegisterOptionsTable(self.name,options)
	self:RegisterChatCommand("dlb", "OpenGUI")
	self:RegisterChatCommand("debuglogbackend", "OpenGUI")
	self.blizzardOptionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions(self.name, self.name)
end

function DebugLogBackend:OnEnable()
	self:UpdateFrame()
	LibDebugLog.RegisterCallback(self, 'MessageLogged')
end

function DebugLogBackend:OnDisable()
	LibDebugLog.UnregisterAllCallbacks(self)
end

function DebugLogBackend:OpenGUI()
	InterfaceOptionsFrame_OpenToFrame(self.blizzardOptionsFrame)
end
